@@ -33,14 +33,13 @@
 #include <libtracker-fts/tracker-fts.h>
 #endif
 
-#include <libtracker-db/tracker-db-manager.h>
-#include <libtracker-db/tracker-db-dbus.h>
-#include <libtracker-db/tracker-db-journal.h>
-
 #include "tracker-class.h"
 #include "tracker-data-manager.h"
 #include "tracker-data-update.h"
 #include "tracker-data-query.h"
+#include "tracker-db-manager.h"
+#include "tracker-db-dbus.h"
+#include "tracker-db-journal.h"
 #include "tracker-ontologies.h"
 #include "tracker-property.h"
 #include "tracker-sparql-query.h"
@@ -361,7 +360,7 @@ tracker_data_update_get_new_service_id (void)
 	}
 
 	if (cursor) {
-		if (tracker_db_cursor_iter_next (cursor, &error)) {
+		if (tracker_db_cursor_iter_next (cursor, NULL, &error)) {
 			max_service_id = MAX (tracker_db_cursor_get_int (cursor, 0), max_service_id);
 		}
 
@@ -406,7 +405,7 @@ tracker_data_update_get_next_modseq (void)
 	}
 
 	if (cursor) {
-		if (tracker_db_cursor_iter_next (cursor, &error)) {
+		if (tracker_db_cursor_iter_next (cursor, NULL, &error)) {
 			max_modseq = MAX (tracker_db_cursor_get_int (cursor, 0), max_modseq);
 		}
 
@@ -493,7 +492,7 @@ cache_insert_value (const gchar            *table_name,
 
 	property.value = *value;
 	property.graph = graph;
-#if TRACKER_HAVE_FTS
+#if HAVE_TRACKER_FTS
 	property.fts = fts;
 #endif
 	property.date_time = date_time;
@@ -526,7 +525,7 @@ cache_delete_value (const gchar            *table_name,
 	property.name = field_name;
 	property.value = *value;
 	property.graph = 0;
-#if TRACKER_HAVE_FTS
+#if HAVE_TRACKER_FTS
 	property.fts = fts;
 #endif
 	property.date_time = date_time;
@@ -1348,6 +1347,18 @@ string_to_gvalue (const gchar         *value,
 	}
 }
 
+static gboolean
+resource_in_domain_index_class (TrackerClass *domain_index_class)
+{
+	guint i;
+	for (i = 0; i < resource_buffer->types->len; i++) {
+		if (g_ptr_array_index (resource_buffer->types, i) == domain_index_class) {
+			return TRUE;
+		}
+	}
+	return FALSE;
+}
+
 static gboolean
 cache_set_metadata_decomposed (TrackerProperty  *property,
                                const gchar      *value,
@@ -1369,7 +1380,7 @@ cache_set_metadata_decomposed (TrackerProperty  *property,
 	super_properties = tracker_property_get_super_properties (property);
 	while (*super_properties) {
 		change |= cache_set_metadata_decomposed (*super_properties, value, value_id,
-		                               graph, graph_id, &new_error);
+		                                         graph, graph_id, &new_error);
 		if (new_error) {
 			g_propagate_error (error, new_error);
 			return FALSE;
@@ -1434,12 +1445,34 @@ cache_set_metadata_decomposed (TrackerProperty  *property,
 		g_value_unset (&gvalue);
 
 	} else {
+
 		cache_insert_value (table_name, field_name, &gvalue,
 		                    graph != NULL ? ensure_resource_id (graph, NULL) : graph_id,
 		                    multiple_values,
 		                    tracker_property_get_fulltext_indexed (property),
 		                    tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_DATETIME);
 
+		if (!multiple_values) {
+			TrackerClass **domain_index_classes;
+
+			domain_index_classes = tracker_property_get_domain_indexes (property);
+			while (*domain_index_classes) {
+				if (resource_in_domain_index_class (*domain_index_classes)) {
+					GValue gvalue_copy = { 0 };
+
+					g_value_init (&gvalue_copy, G_VALUE_TYPE (&gvalue));
+					g_value_copy (&gvalue, &gvalue_copy);
+
+					cache_insert_value (tracker_class_get_name (*domain_index_classes), field_name, &gvalue_copy,
+					                    graph != NULL ? ensure_resource_id (graph, NULL) : graph_id,
+					                    multiple_values,
+					                    tracker_property_get_fulltext_indexed (property),
+					                    tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_DATETIME);
+				}
+				domain_index_classes++;
+			}
+		}
+
 		change = TRUE;
 	}
 
@@ -1491,6 +1524,24 @@ delete_metadata_decomposed (TrackerProperty  *property,
 		                    tracker_property_get_fulltext_indexed (property),
 		                    tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_DATETIME);
 
+		if (!multiple_values) {
+			TrackerClass **domain_index_classes;
+
+			domain_index_classes = tracker_property_get_domain_indexes (property);
+
+			while (*domain_index_classes) {
+				if (resource_in_domain_index_class (*domain_index_classes)) {
+					GValue gvalue_copy = { 0 };
+					g_value_init (&gvalue_copy, G_VALUE_TYPE (&gvalue));
+					g_value_copy (&gvalue, &gvalue_copy);
+					cache_delete_value (tracker_class_get_name (*domain_index_classes), field_name, &gvalue_copy, multiple_values,
+					                    tracker_property_get_fulltext_indexed (property),
+					                    tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_DATETIME);
+				}
+				domain_index_classes++;
+			}
+		}
+
 		change = TRUE;
 	}
 
@@ -1599,6 +1650,25 @@ cache_delete_resource_type (TrackerClass *class,
 			cache_delete_value (table_name, field_name, &gvalue, multiple_values,
 			                    tracker_property_get_fulltext_indexed (prop),
 			                    tracker_property_get_data_type (prop) == TRACKER_PROPERTY_TYPE_DATETIME);
+
+
+			if (!multiple_values) {
+				TrackerClass **domain_index_classes;
+
+				domain_index_classes = tracker_property_get_domain_indexes (prop);
+				while (*domain_index_classes) {
+					if (resource_in_domain_index_class (*domain_index_classes)) {
+						GValue gvalue_copy = { 0 };
+						g_value_init (&gvalue_copy, G_VALUE_TYPE (&gvalue));
+						g_value_copy (&gvalue, &gvalue_copy);
+						cache_delete_value (tracker_class_get_name (*domain_index_classes), field_name, &gvalue_copy, multiple_values,
+						                    tracker_property_get_fulltext_indexed (prop),
+						                    tracker_property_get_data_type (prop) == TRACKER_PROPERTY_TYPE_DATETIME);
+					}
+					domain_index_classes++;
+				}
+			}
+
 		}
 	}
 
@@ -2540,10 +2610,12 @@ ontology_transaction_end (GList *ontology_queue,
 	GList *l;
 	const gchar *ontology_uri = NULL;
 
+	tracker_data_ontology_process_changes_pre_db (seen_classes, seen_properties);
+
 	/* Perform ALTER-TABLE and CREATE-TABLE calls for all that are is_new */
 	tracker_data_ontology_import_into_db (TRUE);
 
-	tracker_data_ontology_process_changes (seen_classes, seen_properties);
+	tracker_data_ontology_process_changes_post_db (seen_classes, seen_properties);
 
 	for (l = ontology_queue; l; l = l->next) {
 		QueuedStatement *queued = ontology_queue->data;
@@ -2564,6 +2636,8 @@ ontology_transaction_end (GList *ontology_queue,
 
 	}
 
+	tracker_data_ontology_process_changes_post_import (seen_classes, seen_properties);
+
 	/* Update the nao:lastModified in the database */
 	if (ontology_uri) {
 		TrackerOntology *ontology;
@@ -2677,7 +2751,7 @@ tracker_data_replay_journal (GHashTable          *classes,
 			tracker_db_journal_reader_get_resource (&id, &uri);
 
 			if (in_ontology) {
-				g_hash_table_insert (id_uri_map, GINT_TO_POINTER (id), (gpointer) uri);
+				g_hash_table_insert (id_uri_map, GINT_TO_POINTER (id), g_strdup (uri));
 				continue;
 			}
 
