@@ -8,7 +8,7 @@
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.          See the GNU
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public
@@ -30,10 +30,13 @@
 
 /* Default values */
 #define DEFAULT_VERBOSITY 0
+#define DEFAULT_MAX_BYTES 1048576   /* 1Mbyte */
+#define ABSOLUTE_MAX_BYTES 10485760 /* 10 Mbytes (GB#616845) */
 
 typedef struct {
 	/* General */
 	gint verbosity;
+	gint max_bytes;
 } TrackerConfigPrivate;
 
 typedef struct {
@@ -63,11 +66,13 @@ enum {
 	PROP_0,
 
 	/* General */
-	PROP_VERBOSITY
+	PROP_VERBOSITY,
+	PROP_MAX_BYTES
 };
 
 static ObjectToKeyFile conversions[] = {
 	{ G_TYPE_INT,     "verbosity",          GROUP_GENERAL,  "Verbosity"       },
+	{ G_TYPE_INT,     "max-bytes",          GROUP_GENERAL,  "MaxBytes"        },
 };
 
 G_DEFINE_TYPE (TrackerConfig, tracker_config, TRACKER_TYPE_CONFIG_FILE);
@@ -93,6 +98,16 @@ tracker_config_class_init (TrackerConfigClass *klass)
 	                                                   DEFAULT_VERBOSITY,
 	                                                   G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
+	g_object_class_install_property (object_class,
+	                                 PROP_MAX_BYTES,
+	                                 g_param_spec_int ("max-bytes",
+	                                                   "Max Bytes",
+	                                                   " Maximum number of UTF-8 bytes to extract per file [0->10485760]",
+	                                                   0,
+	                                                   ABSOLUTE_MAX_BYTES,
+	                                                   DEFAULT_MAX_BYTES,
+	                                                   G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
 	g_type_class_add_private (object_class, sizeof (TrackerConfigPrivate));
 }
 
@@ -105,7 +120,7 @@ static void
 config_set_property (GObject      *object,
                      guint         param_id,
                      const GValue *value,
-                     GParamSpec           *pspec)
+                     GParamSpec   *pspec)
 {
 	switch (param_id) {
 		/* General */
@@ -114,6 +129,11 @@ config_set_property (GObject      *object,
 		                              g_value_get_int (value));
 		break;
 
+	case PROP_MAX_BYTES:
+		tracker_config_set_max_bytes (TRACKER_CONFIG (object),
+		                              g_value_get_int (value));
+		break;
+
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -136,6 +156,10 @@ config_get_property (GObject    *object,
 		g_value_set_int (value, priv->verbosity);
 		break;
 
+	case PROP_MAX_BYTES:
+		g_value_set_int (value, priv->max_bytes);
+		break;
+
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -317,3 +341,34 @@ tracker_config_set_verbosity (TrackerConfig *config,
 	priv->verbosity = value;
 	g_object_notify (G_OBJECT (config), "verbosity");
 }
+
+
+gint
+tracker_config_get_max_bytes (TrackerConfig *config)
+{
+	TrackerConfigPrivate *priv;
+
+	g_return_val_if_fail (TRACKER_IS_CONFIG (config), DEFAULT_MAX_BYTES);
+
+	priv = TRACKER_CONFIG_GET_PRIVATE (config);
+
+	return priv->max_bytes;
+}
+
+void
+tracker_config_set_max_bytes (TrackerConfig *config,
+                              gint           value)
+{
+	TrackerConfigPrivate *priv;
+
+	g_return_if_fail (TRACKER_IS_CONFIG (config));
+
+	if (!tracker_keyfile_object_validate_int (config, "max-bytes", value)) {
+		return;
+	}
+
+	priv = TRACKER_CONFIG_GET_PRIVATE (config);
+
+	priv->max_bytes = value;
+	g_object_notify (G_OBJECT (config), "max-bytes");
+}
