@@ -1,7 +1,6 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2006, Mr Jamie McCracken (jamiemcc@gnome.org)
- * Copyright (C) 2008, Nokia
+ * Copyright (C) 2006, Jamie McCracken <jamiemcc@gnome.org>
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -26,15 +25,16 @@
 
 #include <dbus/dbus-glib-bindings.h>
 
+#include "tracker-status.h"
+
 G_BEGIN_DECLS
 
-gboolean    tracker_dbus_init                    (void);
-void        tracker_dbus_shutdown                (void);
-gboolean    tracker_dbus_register_objects        (void);
-
-GObject    *tracker_dbus_get_object              (GType type);
-void        tracker_dbus_indexer_check_is_paused (void);
-DBusGProxy *tracker_dbus_indexer_get_proxy       (void);
+gboolean        tracker_dbus_init                    (void);
+void            tracker_dbus_shutdown                (void);
+gboolean        tracker_dbus_register_objects        (void);
+GObject        *tracker_dbus_get_object              (GType type);
+TrackerStatus  *tracker_dbus_register_notifier       (void);
+void            tracker_dbus_set_available           (gboolean available);
 
 G_END_DECLS
 
