@@ -232,6 +232,7 @@ update_blank_callback (GPtrArray *blank_nodes,
 
 static gpointer
 query_inthread (TrackerDBCursor *cursor,
+                GCancellable    *cancellable,
                 GError          *error,
                 gpointer         user_data)
 {
@@ -268,7 +269,7 @@ query_inthread (TrackerDBCursor *cursor,
 	column_offsets = g_malloc (n_columns * sizeof (gint));
 	column_data = g_malloc (n_columns * sizeof (gchar*));
 
-	while (tracker_db_cursor_iter_next (cursor, &loop_error)) {
+	while (tracker_db_cursor_iter_next (cursor, cancellable, &loop_error)) {
 		gint i;
 		guint last_offset = -1;
 
@@ -374,7 +375,7 @@ steroids_query (TrackerSteroids *steroids,
 		                          __FUNCTION__);
 
 		reply = dbus_message_new_error_printf (message,
-		                                       DBUS_ERROR_UNKNOWN_METHOD,   
+		                                       DBUS_ERROR_UNKNOWN_METHOD,
 		                                       UNKNOWN_METHOD_MESSAGE,
 		                                       "Query",
 		                                       dbus_message_get_signature (message),
@@ -616,7 +617,7 @@ steroids_update (TrackerSteroids *steroids,
 	} else {
 		tracker_store_sparql_update (query,
 		                             batch ? TRACKER_STORE_PRIORITY_LOW : TRACKER_STORE_PRIORITY_HIGH,
-		                             FALSE,
+		                             batch,
 		                             update_callback,
 		                             sender,
 		                             info,
@@ -636,9 +637,6 @@ tracker_steroids_connection_filter (DBusConnection *connection,
 	g_return_val_if_fail (connection != NULL, DBUS_HANDLER_RESULT_NOT_YET_HANDLED);
 	g_return_val_if_fail (message != NULL, DBUS_HANDLER_RESULT_NOT_YET_HANDLED);
 
-	steroids = user_data;
-	g_return_val_if_fail (TRACKER_IS_STEROIDS (steroids), DBUS_HANDLER_RESULT_NOT_YET_HANDLED);
-
 	if (g_strcmp0 (TRACKER_STEROIDS_PATH, dbus_message_get_path (message))) {
 		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 	}
@@ -647,6 +645,11 @@ tracker_steroids_connection_filter (DBusConnection *connection,
 		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 	}
 
+	/* Only check if the user_data is our TrackerSteroids AFTER having checked that
+	 * the message matches expected path and interface. */
+	steroids = user_data;
+	g_return_val_if_fail (TRACKER_IS_STEROIDS (steroids), DBUS_HANDLER_RESULT_NOT_YET_HANDLED);
+
 	if (!g_strcmp0 ("Query", dbus_message_get_member (message))) {
 		steroids_query (steroids, connection, message);
 		return DBUS_HANDLER_RESULT_HANDLED;
